class IParser:
    pass

class Parser(IParser):
    def __init__(self):
        super().__init__()
        
class MessageParser(Parser):
    def __init__(self):
        super().__init__()
        
    def parse(self, data):
        pass